'' RR20120226  modify for DS1340C

'' ******************************************************************************
'' * DS1340 Object                                                              *
'' * James Burrows May 2006                                                     *
'' * Version 1.2                                                                *
'' ******************************************************************************
''
'' This object provides and example of use of the DS1340 i2c real time clock (RTC)
'' See - for reference:   http://www.maxim-ic.com/quick_view2.cfm/qv_pk/2688
''
'' this object provides the PUBLIC functions:
''  -> Init  - sets up the address and inits sub-objects such
''  -> geti2cError - returns the error passthru from the i2c sub object
''  -> settime - sets the clock time
''  -> setdate - sets the date
''  -> gettime - gets the time into the object - use getday, getmonth etc to read the variables
''  -> getdate - gets the date into the object - see above
''  -> getday/getmonth etc - returns the data got when you call gettime/getdate
''
'' this object provides the PRIVATE functions:
''  -> i2c2bcd - performs integer to BCD conversion
''  -> bcd2int - performs BCD to integer conversion
''
'' this object uses the following sub OBJECTS:
''  -> i2cObject
''
'' Revision History:
''  -> V1 - Release
''      -> V1.1 - Documentation update, slight code tidy-up
''                Changed to include a start status
''                Changed to stop object initializing if device not present on i2cBus
''      -> V1.2 - Updated to allow i2cSCL line driving pass-true to i2cObject
''
'' The default address is %1101_0000

CON

VAR
  long  DS1340_Address
  long  DS1340_Seconds
  long  DS1340_Minutes
  long  DS1340_Hours
  long  DS1340_Days
  long  DS1340_Date
  long  DS1340_Months    
  long  DS1340_Years
  long  started
   
OBJ
  i2cObject     : "i2cObject"

  
PUB Init(_deviceAddress,_i2cSDA,_i2cSCL,_driveSCLLine): okay
  DS1340_Address := _deviceAddress
  i2cObject.init(_i2cSDA, _i2cSCL,_driveSCLLine)

  ' start
  okay := start
  
  return okay

PUB start : okay
  ' start the object
  if started == false
    if i2cObject.devicePresent(DS1340_Address) == true
      started := true
    else
      started := false
  return started     

PUB stop
  ' stop the object
  if started == true
    started := false
    
PUB isStarted : result
  return started

PUB geti2cError : errorCode
  return i2cObject.getError

PUB setTime(ds_hour, ds_minute, ds_seconds)
  if started == true
    ' set the time
    i2cObject.i2cStart
    i2cObject.i2cWrite (DS1340_Address | 0,8)
    i2cObject.i2cwrite(0,8)
    DS1340_Hours   := int2bcd(ds_hour)
    DS1340_Minutes := int2bcd(ds_minute)
    DS1340_Seconds := int2bcd(ds_seconds)  
    i2cObject.i2cWrite (DS1340_Seconds,8)
    i2cObject.i2cWrite (DS1340_Minutes,8)
    i2cObject.i2cWrite (DS1340_Hours,8)        
    i2cObject.i2cStop
  

PUB setDate(ds_day,ds_date,ds_month,ds_year)
  if started == true
  ' set the date 
    i2cObject.i2cStart
    i2cObject.i2cWrite (DS1340_Address | 0,8)
    i2cObject.i2cwrite(3,8)
    DS1340_Days   := int2bcd(ds_day)
    DS1340_Date   := int2bcd(ds_date)
    DS1340_Months := int2bcd(ds_month)
    DS1340_Years  := int2bcd(ds_year)      
    i2cObject.i2cWrite (DS1340_Days,8)  
    i2cObject.i2cWrite (DS1340_Date,8)
    i2cObject.i2cWrite (DS1340_Months,8) 
    i2cObject.i2cWrite (DS1340_Years,8) 
    i2cObject.i2cStop    

  
PUB getDate : ds_seconds | ackbit
  ' get the date bytes from the clock
  if started == true
    i2cObject.i2cStart
    i2cObject.i2cWrite (DS1340_Address | 0,8)
    i2cObject.i2cwrite(3,8)
    i2cObject.i2cStart
    i2cObject.i2cWrite (DS1340_Address | 1,8)
    DS1340_Days   := i2cObject.i2cRead(i2cObject#_i2cACK)
    DS1340_Date   := i2cObject.i2cRead(i2cObject#_i2cACK)
    DS1340_Months := i2cObject.i2cRead(i2cObject#_i2cACK)
    DS1340_Years  := i2cObject.i2cRead(i2cObject#_i2cNAK)
    i2cObject.i2cStop

  
PUB getTime : ds_seconds | ackbit
  ' get the time bytes from the clock
  if started == true
    i2cObject.i2cStart
    i2cObject.i2cWrite (DS1340_Address | 0,8)
    i2cObject.i2cwrite(0,8)
    i2cObject.i2cStart
    i2cObject.i2cWrite (DS1340_Address | 1,8)
    DS1340_Seconds := i2cObject.i2cRead(i2cObject#_i2cACK)
    DS1340_Minutes := i2cObject.i2cRead(i2cObject#_i2cACK)
    DS1340_Hours   := i2cObject.i2cRead(i2cObject#_i2cNAK)
    i2cObject.i2cStop
    return bcd2int(DS1340_Seconds)    
  
  
PUB getHours : result
  return bcd2int(DS1340_Hours)

PUB getMinutes : result
  return bcd2int(DS1340_Minutes)

PUB getSeconds : result
  return bcd2int(DS1340_Seconds)  

PUB getDays : result
  return bcd2int(DS1340_Days)

PUB getDD : result
  return bcd2int(DS1340_Date)

PUB getMonths : result
  return bcd2int(DS1340_Months)

PUB getYears : result
  return bcd2int(DS1340_Years)      
  
pri int2bcd(value) : result
  ' convert integer to BCD (Binary Coded Decimal)
  result := ((value / 10) *16) + (value // 10) 
  return result

pri bcd2int(value) : result
  ' convert BCD (Binary Coded Decimal) to Integer
  result :=((value / 16) *10) + (value // 16) 
  return result 